#!/bin/bash
################################################################
# Creado por Carlos Luis Sanchez Bocanegra Enero 2006
# Modificado el 29/05/2006 para añadirle más funcionalidades
# carlosl.sanchez@juntadeandalucia.es
# Modificado el 15/09/2008 para volcar al log de tamaños de
# Modificado el 30/11/2010 para limpiarlo, se modularizan las funciones
# archivos
# Version 1.1.0
# Para Ares
# Si se añade el parámetro --express, sólo hace los volcados de las bases
# de datos
# Version 1.2.0 de 5/2/2011
# Modificación y adaptación hecha por carlos Luis Sánchez Bocanegra
# Adaptación para poder realizar backups en el servidor norut.no
##############################################################

# Definiciones globales
# Este servicar para darle nombre a las carpetas y acceder a la carpeta del host
CABECERA="bb4"
# Esta determinará cual es el host remoto por si hay carpetas con ese nombre
REMOTO="myhealthservice.itek.norut.no"
# Ruta general de backup
RUTABACKUP="/saluteca/backup/bb4"
# el log
RUTALOG="$RUTABACKUP/log"
#RUTA="$RUTABACKUP/$CABECERA"
# Ruta de backup de las bbdd 
RUTABBDD="$RUTABACKUP/bbdd"
# ruta de backup de los ficheros no bbdd
RUTAFILE="$RUTABACKUP/files"
#RUTACLONE=/cicema/backup/salvaslx/$CLONE/
#BACK=$RUTABACKUP/backup.txt
LOG="$RUTALOG/backup.txt"
STATS="$RUTABACKUP/bat/esttam.pl"
########################clave de mysql ###############################
# Por favor poner permitos 600 root:root en este archivo!!!! sino te corto los huevos
MYSQL_PWD="CLAVE"
################################################################
HISTORICO_DIAS="+7"     # Número de días del histórico de backups anteriores
modo=$1 #Recogo la entrada de parámetros al comando.

##########################################################
## FUNCIONES GENÉRICAS				  ##
##########################################################

# Función que verifica si se han creado el archivo
#       $1 que contiene el archivo a comprobar su existencia
#       $2 el destino backup.txt del log a guardar.
function verifica {
	test -e $1
	if [ -f $? ]; then
		echo "Error en la copia $1" >> $2
	else
		echo "Copia $1 realizada correcta con hash: " >> $2
		md5sum $1 >> $2
	fi
}


# Vuelca un esquema de mysql a un sql comprimido
# $1 : sufijo del archivo comprimido a crear
# $2 : nombre del esquema sql a volcar
function dbdump {
    ts=`date +'%Y%m%d%H%M'`;
    archiv="$RUTABBDD/$CABECERA.$1.$ts.sql.gz"
    mysqldump --databases -l -c -C --host=localhost --password=$MYSQL_PWD --user=root $2 | gzip >$archiv
    verifica $archiv $LOG
}

# Vuelca a un directorio a un fichero comprimido
# $1 : sufijo del archivo comprimido a crear
# $2 : ruta del directorio a volcar
function dirdump {
    ts=`date +'%Y%m%d%H%M'`
    archiv="$RUTAFILE/$1.$ts.tar.gz"
    echo "!!Creando copia de /$2 ..." >> $LOG
    tar cfz $archiv $2
    verifica $archiv $LOG
}

################################################################
## Recogiendo informacion general de entorno ares	     ##
################################################################
#URL=/cicema/www/backup/resultado.html
echo "Fecha y hora inicio:" > $LOG
date >> $LOG
echo >> $LOG
echo "Usuario de copia es `whoami`." >> $LOG
echo "Directorio actual es `pwd`." >> $LOG
echo "Usuarios conectados ahora mismo `who |wc -l`." >> $LOG
echo "Espacio usado en discos `du / -h --max-depth=1`." >> $LOG
echo "Espacio usado en carpetas cicema `du /saluteca/ -h --max-depth=3`." >> $LOG
echo "Analizando logs .......`log_analysis -sa`." >> $LOG
echo "Revisando repositorio sistema ....`apt-get update`." >> $LOG
echo "Actualizando sistema.....`apt-get dist-upgrade -y`." >> $LOG
echo "Limpiando cache......`apt-get clean`." >> $LOG
echo "Actualizando base datos sistema ....`updatedb`." >> $LOG

################################################################
#     Recopilando información para sincronizar con OCSNG      ##
################################################################
#echo "!!Actualizando inventario...... " >> $LOG
#/cicema/ocsagent/ocsinventory-client.pl >> $LOG
#echo "--Finalizado.... " >> $LOG

##################################################################
##		  Optimizando las bases de datos		##
##################################################################
if [ "$modo" != "--express" ]; then
    echo >> $LOG
    echo "!!Optimizando las bases de datos de $CABECERA, previamente a la copia de seguridad" >> $LOG
    mysqlcheck --all-databases --user=root --password=$MYSQL_PWD -h localhost >> $LOG
    echo "--Optimización terminada..." >> $LOG
    echo "Obteniendo información de particionado" >> $LOG
    mount >> $LOG
fi

##################################################################
##      Borrando los archivos más antiguos del histórico	##
##################################################################
find $RUTA -type f -mtime $HISTORICO_DIAS -exec rm -f {} \;


##################################################################
##		Volcando las bases de datos		        ##
##################################################################
# ESQUEMA GLOBAL
echo "!!!Creando copia de seguridad de Mysql en myhealthservice.itek.norut.no" >> $LOG
archiv="$RUTABBDD/$CABECERA.mysqlTOTAL.sql.gz"
mysqldump --all-databases -l -C --host=localhost --password=$MYSQL_PWD --user=root | gzip > $archiv
verifica $archiv $LOG

###############################################
# ESQUEMAS DE MYSQL                            #
###############################################
declare -a db_array
t=0

#Este bcucle recorre cada database y lo dumpea, escluyendo alguna información no necesaria
for i in $( mysqlshow -uroot -p$MYSQL_PWD ); do
  if [ `echo $i | grep -v "Databases" | egrep "^[a-zA-Z0-9]" | grep -v "information_schema"` ]; then
    db_array[$t]=${i};
    dbdump "${db_array[$t]}" "${db_array[$t]}" 
    t=`expr $t + 1`;
  fi
done

####################################################################################################
## VOLCANDO DIRECTORIOS DE DATOS
####################################################################################################
cd /
if [ "$modo" != "--express" ]; then
    ## COPIANDO ESTRUCTURA DE /etc
    dirdump "$CABECERA.etc" "etc"
    ## logs
    dirdump "$CABECERA.logs" "var/log"
    # COPIADO DE LA CARPETA DE TOMCATS 
    dirdump "$CABECERA.tomcat6" "var/lib/tomcat6"
    dirdump "$CABECERA.tomcat6_cache" "var/cache/tomcat6"

    # voy a recorrer todo saluteca y excluir backup y $REMOTO DEL BACKUP
    for i in $( ls saluteca ); do
        if [ `echo $i | grep -v "backup" | grep -v "$REMOTO"` ]; then
		dirdump "$CABECERA.${i}" "saluteca/${i}"
        fi
    done
fi

####################################################################################################
## VOLCANDO EL LOG DE TAMAÑOS DE ARCHIVOS
####################################################################################################
echo " >> Tamaños de archivos:" >> $LOG
$STATS -e ${RUTA}/*.gz >> $LOG
$STATS    ${RUTA}/*.gz >> $RUTALOG/salvaslx.log

####################################################################################################
## Sincronizando ficheros de backup con la FAS
####################################################################################################
#$RUTA_RSYNC --express

####################################################################################################
## RECOGIENDO INFORMACION DE RESTO DE SERVIDORES
####################################################################################################
echo "!!Recogiendo ficheros de myhealthservice" >> $LOG
cd /saluteca/backup
tree >> $LOG
cd /
echo "Fecha Fin de copia de seguridad" >> $LOG
date>> $LOG
exit 0
